<?php

/*
 * Conventions inside functions:
 *  $obj_fullname - string in format "module.object" or "module".
 *  $mod_name   - string in format "module"
 *  $obj_name   - string in format "object"
 */

/**
 * Object loading error and status messages.
 * @var array $loadstatuses
 * @package CTLF
 */
global $loadstatuses;
$loadstatuses = array(
'object not checked'    => 'Object loading failed before checking if object "%obj_fullname%" exists.',
'object does not exist' => 'The object "%obj_fullname%" was not found.',
'object exists'         => 'Object loading stopped after checking if object "%obj_fullname%" exists.',
'config loading error'  => 'Failed to load object "%obj_fullname%" config.',
'config loaded'         => 'Object "%obj_fullname%" loading stopped after loading config.',
'object loading error'  => 'Failed to load object "%obj_fullname%".',
'o.l.e. no load info'   => 'Failed to load object "%obj_fullname%". Loading info in config is missing for object "%obj_fullname%".',
'o.l.e. no mod dir'     => 'Failed to load object "%obj_fullname%". Module directory does not exist.',
'o.l.e. no file'        => 'Failed to load object "%obj_fullname%". A class/function file does not exist.',
'o.l.e. not object from func'   => 'Failed to load object "%obj_fullname%". Loader function returned non-object.',
'o.l.e. not object from class'  => 'Failed to load object "%obj_fullname%". Loader class returned non-object.',
'o.l.e. unknown load type'      => 'Failed to load object "%obj_fullname%". Unknown loader type.',
'object loaded'         => 'Object "%obj_fullname%" has been loaded successfully.',
'unload error'          => 'Object "%obj_fullname%" failed to unload.',
'object unloaded'       => 'Object "%obj_fullname%" is unloaded. Failed to load the object again with an unknown error.',
'unknown'               => 'Unknown error while loading object "%obj_fullname%".');

/**
 * Loaded object array.
 * @var array $modules
 * @package CTLF
 */
global $modules;
$modules = array();

/**
 * Object loading status array.
 * @var array $modstatus
 * @package CTLF
 */
global $modstatus;
$modstatus = array();

/**
 * Object and module configuration.
 * @var array $modconfig
 * @package CTLF
 */
global $modconfig;
if (!isset($modconfig) || !is_array($modconfig) || !sizeof($modconfig))
    $modconfig = array();


//------------------------------------------------------------------------------

/**
 * Loads library.
 *
 * @function loadLibrary
 * @package CTLF
 * @return boolean TRUE on success, FALSE on failure. 
 * @param string $file_name Library file name relative to {@link LIB_DIR}.  
 */  
function loadLibrary($file_name)
{
    //$file_name = LIB_DIR.$file_name;
    //echo '<p style="font-weight: bold;">loadLibrary('.$file_name.')</p>';
    
    if (is_readable(LIB_DIR.$file_name))
    {
        include_once LIB_DIR.$file_name;
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}


//------------------------------------------------------------------------------


/**
 * Fixes incomplete object names. Full object name must be in format "modulename.objectname"
 *
 * @function fixObjectFullname
 * @package CTLF
 * @return string Full object name. 
 * @param string $obj_fullname Object name.  
 */  
function fixObjectFullname($obj_fullname)
{
    // fix "mod" to "mod.mod"
    if (!strpos($obj_fullname, '.'))
        $obj_fullname = $obj_fullname.'.'.$obj_fullname;
    
    return $obj_fullname;
}


/**
 * Returns module name part of the object full name.
 *
 * @function getModuleName
 * @package CTLF
 * @return string Module name. 
 * @param string $obj_fullname Object full name (string in format "modulename.objectname").  
 */  
function getModuleName($obj_fullname)
{
    // return first element before '.'
    return array_shift(explode('.', $obj_fullname));
}


/**
 * Returns object name part of the object full name.
 *
 * @function getObjectName
 * @package CTLF
 * @return string Object name. 
 * @param string $obj_fullname Object full name (string in format "modulename.objectname").  
 */  
function getObjectName($obj_fullname)
{
    // split "module.object" into array("module","object")
    $arr = explode('.', $obj_fullname);
    
    if (sizeof($arr) < 2)
        return $arr[0]; // if only one element, return it
    else
        return $arr[1]; // otherwise return second element
}


/**
 * Finds module directory by given object full name.
 *
 * @function getModuleDir
 * @package CTLF
 * @return mixed Directory name string on success, FALSE on failure. 
 * @param string $obj_fullname Object full name (string in format "modulename.objectname").  
 * @use $modconfig  
 */ 
function getModuleDir($obj_fullname)
{
    global $modconfig;
    
    $mod_name = getModuleName($obj_fullname);
    
    if (isset($modconfig[$obj_fullname]['load']['dir']))
        return $modconfig[$obj_fullname]['load']['dir'];
    else if (isset($modconfig[$mod_name]['load']['dir']))
        return $modconfig[$mod_name]['load']['dir'];
    else if (file_exists(MODULE_DIR.$mod_name) && is_dir(MODULE_DIR.$mod_name))
        return MODULE_DIR.$mod_name;
    else
        return FALSE;
}


/**
 * Checks if the module is already loaded.
 *
 * @function isObjectLoaded
 * @package CTLF
 * @return boolean TRUE if module is loaded successfully, FALSE otherwise. 
 * @param string $obj_fullname Object full name (string in format "modulename.objectname").  
 * @use $modstatus  
 */ 
function isObjectLoaded($obj_fullname)
{
    return ($GLOBALS['modstatus'][$obj_fullname] === 'object loaded');
}


/**
 * Returns object loading error (or status) message.
 *
 * @function getObjectLoadError
 * @package CTLF
 * @return string Status message text. 
 * @param string $obj_fullname Object full name (string in format "modulename.objectname").  
 * @use $modstatus  
 * @use $loadstatuses 
 */ 
function getObjectLoadError($obj_fullname)
{
    global $modstatus, $loadstatuses;
    
    if (isset($loadstatuses[$modstatus[$obj_fullname]]))
        return str_replace('%obj_fullname%', $obj_fullname, $loadstatuses[$modstatus[$obj_fullname]]);
    else
        return str_replace('%obj_fullname%', $obj_fullname, $loadstatuses['unknown']);
}


/**
 * Loads object.
 *
 * @function loadObject
 * @package CTLF
 * @return ref object Loaded object, or FALSE on error. 
 * @param string $obj_fullname Object name. 
 * @param optional boolean $separate_instance Specifies if a separate instance of the object has to be loaded. This instance is not added into global {@link $modules} array.  
 * @use $config  
 * @use $modconfig 
 * @use $modstatus 
 * @use $modules 
 */ 
function &loadObject($obj_fullname, $separate_instance = FALSE)
{
    // uncomment to see object loading order:
    //echo "<p style=\"font-weight: bold;\">function &loadObject($obj_fullname, $separate_instance)</p>";
    
    global $config, $modconfig, $modstatus, $modules;
    
    // fix "mod" to "mod.mod"
    if (!strpos($obj_fullname, '.'))
        $obj_fullname = $obj_fullname.'.'.$obj_fullname;
    
    // set $modstatus[$obj_fullname] to a value:
    if (!isset($modstatus[$obj_fullname]))
        $modstatus[$obj_fullname] = 'object not checked';
    
    // reference $status variable for nicer code:
    $status = &$modstatus[$obj_fullname];
    
    ///echo '<br>BIG IF';
    
    //-- BIG IF --
    // if already loaded, return the object:
    if ($status === 'object loaded' && !$separate_instance)
    {
        ///echo '<br>already loaded';
        return $modules[$obj_fullname];
    }
    
    // if the system was unable to load object previously:
    else if ($status !== 'object not checked' && $status !== 'object exists' && $status !== 'config loaded')
    {
        ////echo '<br>error status';
        return FALSE;
    }
    
    // if object not yet loaded, let's load it:
    else
    {
        ///echo '<br>BIG ELSE';
        
        // get $mod_name and obj_name
        $mod_name = getModuleName($obj_fullname);
        $obj_name = getObjectName($obj_fullname);
        
        ///echo "<br>mod_name: $mod_name, obj_name: $obj_name";
        
        // check if module exists:
        if ($status === 'object not checked')
        {
            if (isset($modconfig[$obj_fullname]['load']))
            {
                // we assume that directories and files exist because
                // modconfig[obj_fullname] is filled in
                $status = 'object exists';
                ///echo '<br>predefined object load';
            }
            else if (isset($modconfig[$mod_name]['load']))
            {
                $status = 'object exists';
                ///echo '<br>predefined module load';
            }
            else
            {
                $dirname = MODULE_DIR.'/'.$mod_name;
                
                // if module directory not exists:
                if ( !(file_exists($dirname) && is_dir($dirname)) )
                {
                    $status = 'o.l.e. no mod dir';
                    return FALSE;
                }
                
                $func_filename = $dirname.'/'.$mod_name.'_loader.php';
                $obj_class_filename = $dirname.'/'.$obj_name.'.class.php';
                $mod_class_filename = $dirname.'/'.$mod_name.'.class.php';
                
                // check for module loader function
                if (file_exists($func_filename) && is_file($func_filename))
                {
                    $modconfig[$mod_name]['load'] = array(
                        'type' => 'function',
                        'dir'  => $dirname,
                        'file' => $mod_name.'_loader.php',
                        'function_name' => $mod_name.'_loader');
                    $status = 'object exists';
                }
                // check for object class
                else if (file_exists($obj_class_filename) && is_file($obj_class_filename))
                {
                    $modconfig[$obj_fullname]['load'] = array(
                        'type' => 'class',
                        'dir'  => $dirname,
                        'file' => $obj_name.'.class.php',
                        'class_name' => $obj_name);
                    $status = 'object exists';
                }
                // check for module object class
                else if (file_exists($mod_class_filename) && is_file($mod_class_filename))
                {
                    $modconfig[$mod_name]['load'] = array(
                        'type' => 'class',
                        'dir'  => $dirname,
                        'file' => $mod_name.'.class.php',
                        'class_name' => $mod_name);
                    $status = 'object exists';
                }
                else
                {
                    $status = 'object does not exist';
                    return FALSE;
                }
            }                
        } // end of 'object not checked'
        
        // load config if not loaded:
        if ($status === 'object exists')
        {
            // merge config arrays (object config over module config)
            $modconfig[$obj_fullname]['config'] = array_merge(
                    (array) @$config[$mod_name],
                    (array) @$config[$obj_fullname],
                    (array) @$modconfig[$mod_name]['config'],
                    (array) @$modconfig[$obj_fullname]['config']
                    );
            
            ///echo '<hr>';
            ///var_dump($modconfig);
            ///echo '<hr>';
            
            $status = 'config loaded';
        }
        
        // load the module itself:
        if ($status === 'config loaded')
        {
            ///echo '<br>config loaded';
            
            if (isset($modconfig[$obj_fullname]['load']))
            {
                $loadinfo = $modconfig[$obj_fullname]['load'];
                ///echo '<br>object loadinfo used';
            }
            else if (isset($modconfig[$mod_name]['load']))
            {
                $loadinfo = $modconfig[$mod_name]['load'];
                ///echo '<br>module loadinfo used';
                
            }
            else
            {
                $status = 'o.l.e. no load info';
                return FALSE;
            }
            
            ///echo '<hr>';
            ///var_dump($loadinfo);
            ///echo '<hr>';
            
            // try to load function/class file:
            if (!is_readable($loadinfo['dir'].'/'.$loadinfo['file']))
            {
                $status = 'o.l.e. no file';
                return FALSE;
            }
            else
            {
                include_once $loadinfo['dir'].'/'.$loadinfo['file'];
            }
            
            if ($loadinfo['type'] === 'function')
            {
                $obj = &call_user_func($loadinfo['function_name'], $obj_fullname, $modconfig[$obj_fullname]['config']);

                if (is_object($obj))
                {
                    $modules[$obj_fullname] = &$obj;
                    $status = 'object loaded';
                    return $modules[$obj_fullname];
                }
                else
                {
                    $status = 'o.l.e. not object from func';
                    return FALSE;
                }
            }
            else if ($loadinfo['type'] === 'class')
            {
                $class_name = $loadinfo['class_name'];
                ///var_dump($class_name);
                $obj = &new $class_name($obj_fullname, $modconfig[$obj_fullname]['config']);
                if (is_object($obj))
                {
                    $modules[$obj_fullname] = $obj;
                    $status = 'object loaded';
                    return $modules[$obj_fullname];
                }
                else
                {
                    $status = 'o.l.e. not object from class';
                    return FALSE;
                }
            }
            else
            {
                $status = 'o.l.e. unknown load type';
                return FALSE;
            }
        } // end of 'config loaded'
    } // end of else can continue
    // -- END OF BIG IF ---
    
    // in all other cases return failure:
    $status = 'unknown';
    return FALSE;
}


/**
 * Unloads an object.
 *
 * @function unloadObject
 * @package CTLF
 * @return boolean TRUE on success, FALSE on error.
 * @param string $obj_fullname Object full name (string in format "modulename.objectname").
 * @param ref object $object Object to unload.
 * @use $modconfig 
 * @use $modstatus 
 * @use $modules 
 */ 

function unloadObject($obj_fullname, &$object)
{
    global $modstatus, $modules, $modconfig;
    
    // if object already unloaded:
    if ($modstatus[$obj_fullname] === 'object unloaded')
    {
        return TRUE;
    }
    else if ($modstatus[$obj_fullname] !== 'object loaded')
    {
        // cannot unload object since it is not loaded
        return FALSE;
    }
    else
    {
        $mod_name = getModuleName($obj_fullname);
        $load_type = @$modconfig[$mod_name]['load']['type'];
        
        if ($load_type == 'function')
        {
            if (function_exists($mod_name.'_unload'))
            {
                if (call_user_func($mod_name.'_unload', $obj_fullname, $object))
                {
                    // object unloaded successfully:
                    $modstatus[$obj_fullname] = 'object unloaded';
                    return TRUE;
                }
                else
                {
                    // object unloaded unsuccessfully
                    $modstatus[$obj_fullname] = 'unload error';
                    return FALSE;
                }
            }
            else
            {
                // unloader function does not exist (that means the module does not need it).
                return TRUE;
            }
        } // end if ($load_type == function)
        else if ($load_type == 'class')
        {
            if (method_exists($object, 'unload'))
            {
                if (call_user_func(array($object, 'unload'), $obj_fullname))
                {
                    // object unloaded successfully:
                    $modstatus[$obj_fullname] = 'object unloaded';
                    return TRUE;
                }
                else
                {
                    // object unloaded unsuccessfully
                    $modstatus[$obj_fullname] = 'unload error';
                    return FALSE;
                }
            }
            else
            {
                // unloader method does not exist (that means the module does not need it).
                return TRUE;
            }
        } // end if ($load_type == class)
    }
}

?>